unit sStylePassive;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, sConst;
{$DEFINE TSHINTS}
{$DEFINE TSMENUS}

type
  TsPassivePaintStyle = class;
  TsPassiveBackground = class;
  TsSelection = class;

  TsBorder = class(TPersistent)
  private
    FOwner : TsSelection;
    FWidth : integer;
    FBevel : TsBorderStyle;
    procedure SetWidth(const Value: integer);
  public
    constructor Create(AOwner: TsSelection);
  published
    property Width : integer read FWidth write SetWidth default 1;
    property Bevel : TsBorderStyle read FBevel write FBevel default sConst.bsFlat;
  end;

  TsSelection = class(TPersistent)
  private
    FMode : TModeSelection;
    FOwner : TPersistent;
    FBorder : TsBorder;
    procedure SetMode(const Value: TModeSelection);
  public
    constructor Create(AOwner: TPersistent);
    destructor destroy; override;
  published
    property Border : TsBorder read FBorder write FBorder;
    property Mode : TModeSelection read FMode write SetMode default [msBorder, msColor];
  end;

  TPassiveGradient = class(TPersistent)
  private
    FOwner : TsPassiveBackground;
    FColor1 : TColor;
    FColor2 : TColor;
    FLayout   : TGradientTypes;
    procedure SetColors (Index: Integer; Value: TColor);
    procedure SetLayout(const Value: TGradientTypes);
  public
    constructor Create(AOwner: TsPassiveBackground);
    destructor Destroy; override;
  published
    property Color1: TColor index 0 read FColor1 write SetColors default $00E0E0E0;
    property Color2: TColor index 1 read FColor2 write SetColors default clSilver;
    property Layout: TGradientTypes read FLayout write SetLayout   default gtTopBottom;
  end;

  TsPassiveBackground = class(TPersistent)
  protected
    FOwner : TPersistent;//TsPassiveBGStyle;
    FImage : TPicture;
    FGradient : TPassiveGradient;
    FListenMSG : boolean;
    procedure SetImage(const Value: TPicture);
    procedure SetStyle(const Value: TBgStyle);
  public
    FMode : TBgStyle;
    constructor Create(AOwner : TPersistent);//TsPassiveBGStyle);
    destructor Destroy; override;
    procedure BkGroundsUpdate; dynamic;
  published
    property ListenMSG : boolean read FListenMSG write FListenMSG default True;
    property Mode : TBgStyle read FMode write SetStyle default bgsNone;
    property Image : TPicture read FImage write SetImage;
    property Gradient : TPassiveGradient read FGradient write FGradient;
  end;

  TsPassiveShadow = class(TPersistent)
  private
    FEnabled : boolean;
    FOffset : integer;
    FColor : TColor;
    FOwner : TsPassivePaintStyle;
    FTransparency : integer;
    FBlur : integer;

    procedure SetColor(const Value: TColor);
    procedure SetEnabled(const Value: boolean);
    procedure SetOffset(const Value: integer);
    procedure SetTransparency(const Value: integer);
    procedure SetBlur(const Value: integer);
  public
    {  }
    constructor Create(AOwner : TPersistent);
  published
    {   }
    property Transparency: integer read FTransparency write SetTransparency default 70;
    {      }
    property Enabled : boolean read FEnabled write SetEnabled default True;
    {  }
    property Color : TColor read FColor write SetColor default clBlack;
    {    }
    property Offset : integer read FOffset write SetOffset default 10;
    property Blur : integer read FBlur write SetBlur default 4;
  end;

  TsPassivePainting = class(TPersistent)
  private
    FBevel : TPanelBevel;
    FColorActive : Graphics.TColor;
    FColorInActive : Graphics.TColor;
    FParentColor : boolean;
    FColorBorderTop : Graphics.TColor;
    FColorBorderBottom : Graphics.TColor;
    FTransparency: integer;
    FOwner : TsPassivePaintStyle;
    procedure SetColors (Index: Integer; Value: Graphics.TColor);
    procedure SetTransparency(const Value: integer);
    function GetBevel: TPanelBevel;
    procedure SetBevel(const Value: TPanelBevel);
  public
    {  }
    constructor Create(AOwner : TPersistent);
  published
    property Bevel : TPanelBevel read GetBevel write SetBevel default bvRaised;
    property Transparency: integer read FTransparency write SetTransparency default 0;
    {     }
    property ColorBorderTop: Graphics.TColor index 0 read FColorBorderTop write SetColors default scDarkSky;
    {     }
    property ColorBorderBottom: Graphics.TColor index 1 read FColorBorderBottom write SetColors default scDarkSky;
    {   ,   }
    property ColorActive: Graphics.TColor index 2 read FColorActive write SetColors default scLightYellow;
    {   ,    }
    property ColorInActive: Graphics.TColor index 3 read FColorInActive write SetColors default cl3DLight;
  end;

  TsPassivePaintStyle = class(TPersistent)
  protected
    FGroupIndex : integer;                             
  public
    FOwner : TComponent;
    FShadow : TsPassiveShadow;
    FPainting : TsPassivePainting;
    FBackground : TsPassiveBackground;
    FSelection : TsSelection;

    FDrawingStop : boolean;
    FCacheBmp : Graphics.TBitmap;
    COC : integer;
    sC : TComponent;//TsStyleControl;
    constructor Create(AOwner : TComponent); overload;
    destructor Destroy; override;
    procedure OwnerInvalidate;
    procedure WndProc(var Message: TMessage);
    procedure PaintBG(aRect: TsRect);
    procedure PaintBorder(DC : longint; aRect : TsRect; Active : boolean; wLimit: integer);
  published
    property Background : TsPassiveBackground read FBackground write FBackground;
    {   @link(TsShadow)}
    property Shadow: TsPassiveShadow read FShadow write FShadow;
    { ,   }
    property Painting: TsPassivePainting read FPainting write FPainting;
    {  .   @link(TsStyleControl).  -1,     }
    property GroupIndex: integer read FGroupIndex write FGroupIndex default 0;
    property Selection : TsSelection read FSelection write FSelection;
  end;

implementation

uses sGraphUtils, {sVclUtils,} sUtils, sCustomMenuManager{, sHintManager, sStyleUtil};

{ TsBorder }

constructor TsBorder.Create(AOwner: TsSelection);
begin
  FOwner := AOwner;
  FWidth := 1;
  FBevel := sConst.bsFlat;
end;

procedure TsBorder.SetWidth(const Value: integer);
begin
  if FWidth <> Value then begin
    FWidth := Value;
//    FOwner.FOwner.Fowner.Invalidate;
  end;
end;

{ TsSelection }

constructor TsSelection.Create(AOwner: TPersistent);//aintStyle);
begin
  FOwner := AOwner;
  FBorder := TsBorder.Create(Self);
  FMode := [msBorder, msColor];
end;

destructor TsSelection.destroy;
begin
//  FOwner := nil;
  FreeAndNil(FBorder);
  inherited;
end;

procedure TsSelection.SetMode(const Value: TModeSelection);
begin
  if (FMode <> Value) and not (msShadow in Value) then begin
    FMode := Value;
//    FOwner.FOwner.Invalidate;
  end;
end;

{ TPassiveGradient }

constructor TPassiveGradient.Create(AOwner: TsPassiveBackground);
begin
  FOwner := TsPassiveBackground(AOwner);
  FLayout := gtTopBottom;
  FColor1 := $00E0E0E0;
  FColor2 := clSilver;
end;

destructor TPassiveGradient.Destroy;
begin
//  FOwner := nil;
  inherited;
end;

procedure TPassiveGradient.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: FColor1 := ColorToRGB(Value);
    1: FColor2 := ColorToRGB(Value);
  end;
  FOwner.BkGroundsUpdate;
end;

procedure TPassiveGradient.SetLayout(const Value: TGradientTypes);
begin
  FLayout := Value;
  FOwner.BkGroundsUpdate;
end;

{ TsPassiveBackground }

constructor TsPassiveBackground.Create(AOwner: TPersistent);//TsPassiveBGStyle);
begin
  FOwner := AOwner;
  FMode := bgsNone;
  FImage := TPicture.Create;
  FListenMSG := True;
  FGradient := TPassiveGradient.Create(Self);
end;

destructor TsPassiveBackground.Destroy;
begin
//  FOwner := nil;
  FreeAndNil(FImage);
  FreeAndNil(FGradient);
  inherited;
end;

procedure TsPassiveBackground.BkGroundsUpdate;
begin
//  FOwner.FOwner.Invalidate;
end;

procedure TsPassiveBackground.SetImage(const Value: TPicture);
begin
  if FImage <> Value then begin
    FImage.Assign(Value);
    BkGroundsUpdate;
  end;
end;

procedure TsPassiveBackground.SetStyle(const Value: TBgStyle);
begin
  try
//    ShowError(FOwner.FOwner.Name);
    if FMode <> Value then begin
      FMode := Value;
      BkGroundsUpdate;
//      FListenMSG := False;
    end;
  except
  end;
end;

{ TsPassiveShadow }

constructor TsPassiveShadow.Create(AOwner: TPersistent);
begin
  inherited Create;
  FOwner := TsPassivePaintStyle(AOwner);
  FColor := clBlack;
  FBlur := 4;
  FOffset := 10;
  FTransparency := 70;
  FEnabled := True;
end;

procedure TsPassiveShadow.SetBlur(const Value: integer);
begin
  if FBlur <> Value then begin
    FBlur := Value;
  end;
end;

procedure TsPassiveShadow.SetColor(const Value: TColor);
begin
  if (FColor <> Value) then begin
    FColor := Value;
  end;
end;

procedure TsPassiveShadow.SetEnabled(const Value: boolean);
begin
  if (FEnabled <> Value) then begin
    FEnabled := Value;
  end;
end;

procedure TsPassiveShadow.SetOffset(const Value: integer);
begin
  if (FOffset <> Value) then begin
    FOffset := Value;
  end;
end;

procedure TsPassiveShadow.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    if Value < 0 then FTransparency := 0
    else if Value > 100 then FTransparency := 100
    else FTransparency := Value;
  end;
end;

{ TsPassivePainting }

constructor TsPassivePainting.Create(AOwner: TPersistent);
begin
  inherited Create;
  FColorBorderTop := scDarkSky;
  FColorBorderBottom := scDarkSky;
  FColorActive := scLightYellow;
  FColorInActive := scLightBlue;
  FOwner := TsPassivePaintStyle(AOwner);
end;

function TsPassivePainting.GetBevel: TPanelBevel;
begin
  Result := FBevel;
end;

procedure TsPassivePainting.SetBevel(const Value: TPanelBevel);
begin
  if FBevel <> Value then begin
    FBevel := Value;
  end;
end;

procedure TsPassivePainting.SetColors(Index: Integer; Value: TColor);
begin
  case Index of
    0: FColorBorderTop := Value;
    1: FColorBorderBottom := Value;
    2: FColorActive := Value;
    3: begin
      FColorInActive := Value;
    end;
  end;
end;

procedure TsPassivePainting.SetTransparency(const Value: integer);
begin
  if FTransparency <> Value then begin
    FTransparency := Value;
  end;
end;

{ TsPassivePaintStyle }

constructor TsPassivePaintStyle.Create(AOwner : TComponent);
begin
  inherited Create;
  FOwner := AOwner;
  FCacheBmp := Graphics.TBitmap.Create;

  FShadow := TsPassiveShadow.Create(Self);
  FPainting := TsPassivePainting.Create(Self);
  FBackground := TsPassiveBackground.Create(Self);
  Selection := TsSelection.Create(Self);

{$IFDEF ALLOWPROP}
  sC := GetsStyleControl(sVclUtils.GetParentForm(AOwner), GroupIndex);
  if not Assigned(sC) then begin
    FGroupIndex := 0;
  end
  else begin
    Painting.FColorActive := sC.ColorActive;
    Painting.FColorInActive := sC.ColorInActive;
    Painting.FColorBorderTop := sC.ColorBorderTop;
    Painting.FColorBorderBottom := sC.ColorBorderBottom;
    FGroupIndex := sC.GroupIndex;
    Painting.FParentColor := sC.ParentColor;
    Shadow.Color := sC.ShadowColor;
    Shadow.Offset := sC.ShadowOffset;
    Shadow.Transparency := sC.ShadowTransparency;
    Shadow.Blur := sC.ShadowBlur;
    Shadow.Enabled := sC.ShadowEnabled;
  end;
{$ENDIF}
end;

destructor TsPassivePaintStyle.Destroy;
begin
  FreeAndNil(FPainting);
  FreeAndNil(FShadow);
  FreeAndNil(FBackground);
  FreeAndNil(FCacheBmp);
  FreeAndNil(FSelection);
  inherited;
end;                   

procedure TsPassivePaintStyle.OwnerInvalidate;
begin
  Case COC of
    COC_TsHintManager : begin
//      TsHintManager(FOwner).Invalidate;
    end;
    COC_TsCustomMenuManager, COC_TsMenuManager : begin
      TsCustomMenuManager(FOwner).Invalidate;
    end;
  end;
end;

procedure TsPassivePaintStyle.PaintBG(aRect: TsRect);
begin
if WidthOf(aRect) > 40000 then alert;
  case Background.Mode of
    bgsNone, bgsTransparent: begin
      FCacheBmp.Canvas.Pen.Style := psClear;
      FCacheBmp.Canvas.Brush.Style := bsSolid;
      FCacheBmp.Canvas.Brush.Color := Painting.ColorInActive;
      inc(aRect.Right);
      inc(aRect.Bottom);
      FCacheBmp.Canvas.Rectangle(aRect);
    end;
    bgsImage: begin
      if Assigned(Background.Image) and Assigned(Background.Image.Graphic) and (Background.Image.Width <> 0) and (Background.Image.Height <> 0) then begin
        TileBitmap(FCacheBmp.Canvas, aRect, Background.Image.Graphic);
      end;
    end;
    bgsGradient: begin
      case Background.Gradient.Layout of
        gtTopBottom, gtLeftRight : begin
          GradientBmp(FCacheBmp, aRect, Background.Gradient.Color1, Background.Gradient.Color2, Background.Gradient.Layout);
        end;
      end;
    end;
  end;
end;

procedure TsPassivePaintStyle.PaintBorder(DC: Integer; aRect: TsRect; Active: boolean; wLimit: integer);
var
  GetColor : array [0..1] of TColor;
  R: TRect;
  Width, i, n : integer;
  Color1, Color2 : TColor;
  procedure DrawRect; begin
    DrawRectangleOnDC(DC, R, ColorToRGB(Color1), Color2, Width)
  end;
begin
  GetColor[1] := ColorToRGB(Painting.ColorBorderTop);
  GetColor[0] := ColorToRGB(Painting.ColorBorderBottom);
  Color1 := GetColor[1];
  Color2 := GetColor[0];
  R := aRect;
  if Active and (msBorder in Selection.Mode) then begin
    if wLimit > 0
      then n := Mini(wLimit, Selection.Border.Width)
      else n := Selection.Border.Width;
    Case Selection.Border.Bevel of
      bsFlat : begin
        width := n;
        DrawRect;
      end;
      sConst.bsRaised: begin
        for i := 1 to n do begin
          width := 1;
          Color1 := LightingColor(GetColor[1], 200, i / n);
//          Color2 := DarkingColor(GetColor[1], 20, i / n);
          Color2 := LightingColor(GetColor[1], 20, i / n);
          DrawRect;
        end;
      end;
      sConst.bsLowered: begin
        for i := 1 to n do begin
          width := 1;
          Color1 := DarkingColor(GetColor[1], 0, i / n);
          Color2 := DarkingColor(GetColor[1], 255, i / n);
          DrawRect;
        end;
      end;
    end;
  end
  else begin
    width := 1;
    Case Painting.Bevel of
      bvNone : begin
        width := 0;
      end;
      bvRaised : begin
      end;
      bvLowered : begin
        Color1 := GetColor[0];
        Color2 := GetColor[1];
      end;
      bvSpace: begin
        Color1 := GetColor[1];
        Color2 := GetColor[1];
      end
    end;
    DrawRect;
  end;
end;

procedure TsPassivePaintStyle.WndProc(var Message: TMessage);
var
  i : integer;
//  sSC : TsStyleControl;
begin
{$IFDEF ALLOWPROP}
  if (Message.Msg >= SM_OFFSET) and (Message.Msg <= SM_LAST) then begin
    if ((Message.WParam = GroupIndex) or (Message.WParam = GI_FORPANELPATTERN)) then begin
      case Message.Msg of
        SM_SETBORDERTOP:      Painting.ColorBorderTop := TSMSetColor(Message).Value;
        SM_SETBORDERBOTTOM:   Painting.ColorBorderBottom := TSMSetColor(Message).Value;
        SM_SETCOLORACTIVE:    Painting.ColorActive := TSMSetColor(Message).Value;
        SM_SETCOLORINACTIVE:  Painting.ColorInActive := TSMSetColor(Message).Value;
        SM_SETCOLORSHADOW :   Shadow.Color := TSMSetColor(Message).Value;

        SM_SETSHADOWENABLED:  Shadow.Enabled := TSMSetBoolean(Message).Value;
        SM_SHADOWTRANSPARENCY:Shadow.Transparency := TSMSetInteger(Message).Value;
        SM_SHADOWOFFSET:      Shadow.Offset       := TSMSetInteger(Message).Value * 2;
        SM_SHADOWBLUR:        Shadow.Blur         := TSMSetInteger(Message).Value * 2;

        SM_GETSTYLEINFO:      begin
            TSMGetStyleInfo(Message).WParam := 1;
            TSMGetStyleInfo(Message).LParam := Longint(Self);
          end;

        SM_SETPATTERN: begin
          if Background.ListenMSG then begin
            Background.Mode := TSMSetBground(Message).sBackground.Mode;
            case TSMSetBground(Message).sBackground.Mode of
              bgsNone: begin
                Background.Image.Assign(nil);
              end;
              bgsGradient: begin
                Background.Gradient.Color1 := TSMSetBground(Message).sBackground.Gradient.Color1;
                Background.Gradient.Color2 := TSMSetBground(Message).sBackground.Gradient.Color2;
  //                  Background.Gradient.Layout := TSMSetBground(Message).sBackground.Gradient.Layout;
              end;
              bgsImage: begin
                if Assigned(TSMSetBground(Message).sBackground.Image.Bitmap) then begin
                  Background.Image.Bitmap.Assign(TSMSetBground(Message).sBackground.Image.Bitmap);
                end;
              end;
            end;
//            Invalidate;
          end;
          Message.Result := 1;
        end;

        SM_CHANGESTYLECONTROL : begin
{            sSC := TSMChangeStyleControl(Message).sStyleControl;

            Painting.ColorBorderTop := sSC.ColorBorderTop;
            Painting.ColorBorderBottom := sSC.ColorBorderBottom;
            Painting.ColorActive := sSC.ColorActive;
            Painting.ColorInActive := sSC.ColorInActive;
            Shadow.Color := sSC.ShadowColor;
            Shadow.Enabled := sSC.ShadowEnabled;

            Shadow.Offset       := sSC.ShadowOffset * 2;
            Shadow.Blur         := sSC.ShadowBlur * 2;

            if Background.ListenMSG and Assigned(sSC.Background) then begin
              Background.Mode := sSC.Background.Mode;
              case sSC.Background.Mode of
                bgsNone: begin
                  Background.Image.Assign(nil);
                end;
                bgsImage: begin
                  if Assigned(sSC.Background.Image) then begin
                    Background.Image.Assign(sSC.Background.Image);
                  end;
                end;
                bgsGradient: begin
                  Background.Gradient.Color1 := sSC.Background.Gradient.Color1;
                  Background.Gradient.Color2 := sSC.Background.Gradient.Color2;
    //                  Background.Gradient.Layout := sSC.Background.Gradient.Layout;
                end;
              end;
//              FOwner.Invalidate;
            end;
          end;
        }
      end;
    end;
  end;
{$ENDIF}
end;

end.
